# SPDX-FileCopyrightText: Copyright (C) 2025, Antoine Basset
# SPDX-PackageSourceInfo: https://github.com/kabasset/azulero
# SPDX-License-Identifier: Apache-2.0

import argparse
from pathlib import Path
import requests

from azulero.timing import Timer


def add_parser(subparsers):
    parser = subparsers.add_parser("retrieve", help="Retrieve MER datafiles")

    parser.add_argument(
        "tiles",
        type=str,
        nargs="+",
        help="Tile indices",
    )
    parser.add_argument("--dsr", type=str, default="DR1_R1", help="Data set release")

    parser.set_defaults(func=run)


def query_datafiles(tile, dsr):
    print(f"Query datafiles for tile {tile}:")

    query = {
        "project": "EUCLID",
        "class_name": "DpdMerBksMosaic",
        "Data.TileIndex": tile,
        "Header.DataSetRelease": dsr,
        "fields": "Data.DataStorage.DataContainer.FileName:Data.Filter.Name",
    }
    lines = (
        requests.get("https://eas-dps-rest-ops.esac.esa.int/REST", params=query)
        .text.replace('"', "")
        .split()
    )
    datafiles = {}
    for l in lines:
        if "VIS" in l or "NIR" in l:
            file_name, filter_name = l.split(",")
            datafiles[file_name] = filter_name
    for f in datafiles:
        print(f"- [{datafiles[f]}] {f}")
    return datafiles


def make_workdir(workspace, tile):
    workdir = Path(workspace).expanduser() / tile
    if workdir.is_dir():
        print("WARNING: Working directory already exists.")
    else:
        workdir.mkdir(parents=True)
    return workdir


def download_datafiles(datafiles, workdir):
    print(f"Download and extract datafiles to: {workdir}")

    for n in datafiles:  # TODO parallelize?
        path = (workdir / n).with_suffix("")
        if path.is_file():
            print(f"WARNING: File existd; skip: {path.name}")
            continue
        r = requests.get(f"https://euclidsoc.esac.esa.int/{n}")
        with open(path, "wb") as f:
            f.write(r.content)
        print(f"- {path}")


def run(args):

    print()

    timer = Timer()
    for tile in args.tiles:  # TODO parallelize?
        workdir = make_workdir(args.workspace, tile)
        datafiles = query_datafiles(tile, args.dsr)
        timer.tic_print()
        if len(datafiles) < 4:
            print(f"ERROR: Only {len(datafiles)} files found; Skipping this tile.")
            continue
        if len(datafiles) > 4:
            print(f"WARNING: More than 4 files found: {len(datafiles)}.")
        download_datafiles(datafiles, workdir)
        timer.tic_print()
        print(f"\nYou may now run:")
        print(f"\nazul --workspace {args.workspace} show {tile}\n")
        print(f"or:")
        print(f"\nazul --workspace {args.workspace} process {tile}\n")



# SPDX-FileCopyrightText: Copyright (C) 2025, Antoine Basset
# SPDX-PackageSourceInfo: https://github.com/kabasset/azulero
# SPDX-License-Identifier: Apache-2.0

import argparse
import numpy as np
from pathlib import Path

from azulero import color, io, mask
from azulero.timing import Timer


def add_parser(subparsers):
    parser = subparsers.add_parser("process", help="Process MER channels")

    parser.add_argument(
        "tile",
        type=str,
        help="Tile folder name and optional slicing",
    )
    parser.add_argument(
        "--output",
        "-o",
        type=str,
        default="output.tiff",
        help="Output filename relative to the tile folder",
    )
    parser.add_argument(
        "--black",
        type=float,
        default=0.0,
        help="Black point (0 for background-subtracted inputs)",
    )
    parser.add_argument(
        "--white",
        type=float,
        default=10000.0,
        help="White point",
    )
    parser.add_argument(
        "--scaling",
        nargs=4,
        type=float,
        default=[500, 1.6, 1, 1],  # NIR passbands ~ 0.25, 0.4, 0.5, H = 1 boosts R
        help="Scaling factors for IYJH bands",
    )
    parser.add_argument("--nirl", type=float, default=0.5, help="NIR contribution to L")
    parser.add_argument("--yg", type=float, default=0.3, help="Y contribution to G")
    parser.add_argument("--ib", type=float, default=0.2, help="I contribution to B")
    parser.add_argument(
        "--stretch",
        type=float,
        default=0.7,
        help="Stretching parameter (arcsinh scaling factor)",
    )
    parser.add_argument(
        "--saturation", type=float, default=1.6, help="Saturation factor"
    )

    parser.set_defaults(func=run)


def run(args):

    print()

    transform = color.Transform(
        iyjh_scaling=np.array(args.scaling),
        nir_to_l=args.nirl,
        y_to_g=args.yg,
        i_to_b=args.ib,
        saturation=args.saturation,
        stretch=args.stretch,
        bw=np.array([args.black, args.white]),
    )

    tile, slicing = io.parse_tile(args.tile)
    workdir = Path(args.workspace).expanduser() / tile

    timer = Timer()

    print(f"Read IYJH image from: {workdir}")
    iyjh = io.read_iyjh(workdir, slicing)
    print(f"- Shape: {iyjh.shape[1]} x {iyjh.shape[2]}")
    timer.tic_print()

    print(f"Detect invalid pixels")
    dead_vis, dead_nir = mask.dead_pixels(*iyjh)
    hot = mask.hot_pixels(*iyjh)
    print(f"- Dead VIS: {np.sum(dead_vis)}")
    print(f"- Dead NIR: {np.sum(dead_nir)}")
    print(f"- Hot: {np.sum(hot)}")
    timer.tic_print()

    print(f"Transform IYJH to RGB image")
    res = color.iyjh_to_rgb(iyjh, transform)
    del iyjh
    io.write_tiff(res, workdir / "rgb.tiff")
    timer.tic_print()

    print(f"Inpaint invalid pixels")
    res = mask.inpaint(res, dead_nir)
    res = mask.inpaint(res, dead_vis)
    res[dead_vis] = mask.resaturate(res[dead_vis])
    res = mask.inpaint(res, hot)
    timer.tic_print()

    print(f"Write output to: {args.output}")
    io.write_tiff(res, workdir / args.output)
    timer.tic_print()
